package ROUGE;

# ------------------------------------------------------------------------

#Copyright (C) 2005 Enrique Amig and Jess Gimnez

#This library is free software; you can redistribute it and/or
#modify it under the terms of the GNU Lesser General Public
#License as published by the Free Software Foundation; either
#version 2.1 of the License, or (at your option) any later version.

#This library is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#Lesser General Public License for more details.

#You should have received a copy of the GNU Lesser General Public
#License along with this library; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# ------------------------------------------------------------------------

use strict;
use Data::Dumper;
use IQ::IQXML;
use IQ::COMMON;

use vars qw($ROUGEXT $CFGEXT $rROUGE $TROUGE);

$ROUGE::ROUGEXT = "RG";
$ROUGE::TROUGE = "ROUGE/ROUGE-1.5.5.pl";             # version 1.5.5
$ROUGE::TROUGEdata = "ROUGE/data";                   # version 1.5.5
$ROUGE::rROUGE = { "RG-1" => 1, "RG-2" => 1, "RG-3" => 1, "RG-4" => 1, "RG-L" => 1, "RG-S*" => 1, "RG-SU*" => 1, "RG-W-1.2" => 1 };
$ROUGE::CFGEXT = "config";

sub read_rouge {
   #description _ read ROUGE value from report file (for all segments)
   #param1  _ report filename
   #@return _ ROUGE score list

   my $report = shift;

   my %Hrouge;
   my @lrouge;

   open(AUX, "< $report") or die "couldn't open file: $report\n";
   while (my $aux = <AUX>) {
      if ($aux =~ /^X ROUGE-[SLW1-4][^ ]* Average_F.*/) {
         chomp($aux);
         my @laux = split(/ +/, $aux);
         $Hrouge{$laux[1]} = $laux[3];
      }
   }
   close(AUX);

   foreach my $s (sort keys %Hrouge) { push(@lrouge, $Hrouge{$s}); }

   return \@lrouge;
}

sub read_rouge_segments {
   #description _ read ROUGE value from report file (for all segments)
   #param1  _ report filename
   #@return _ ROUGE score list

   my $report = shift;

   my @lrouge1;
   my @lrouge2;
   my @lrouge3;
   my @lrouge4;
   my @lrougeL;
   my @lrougeS;
   my @lrougeSU;
   my @lrougeW;

   open(AUX, "< $report") or die "couldn't open file: $report\n";
   while (my $aux = <AUX>) {
      if ($aux =~ /^X ROUGE-[SLW1-4][^ ]* Eval.*/) {
         chomp($aux);
         my @laux = split(/ +/, $aux);
         my @ll = split(/\./, $laux[3]);
         my @llF = split(":", $laux[6]);
         if ($laux[1] eq "ROUGE-1") { push(@lrouge1, $llF[1]); }
         if ($laux[1] eq "ROUGE-2") { push(@lrouge2, $llF[1]); }
         if ($laux[1] eq "ROUGE-3") { push(@lrouge3, $llF[1]); }
         if ($laux[1] eq "ROUGE-4") { push(@lrouge4, $llF[1]); }
         if ($laux[1] eq "ROUGE-L") { push(@lrougeL, $llF[1]); }
         if ($laux[1] eq "ROUGE-S*") { push(@lrougeS, $llF[1]); }
         if ($laux[1] eq "ROUGE-SU*") { push(@lrougeSU, $llF[1]); }
         if ($laux[1] eq "ROUGE-W-1.2") { push(@lrougeW, $llF[1]); }
      }
   }
   close(AUX);

   my @lSEG = (\@lrouge1, \@lrouge2, \@lrouge3, \@lrouge4, \@lrougeL, \@lrougeS, \@lrougeSU, \@lrougeW);

   return \@lSEG;
}

sub computeMultiROUGE {
   #description _ computes ROUGE scores -> n = 1..4, LCS, S*, SU*, W-1.2 (multiple references)
   #param1 _ source file
   #param2 _ candidate file
   #param3 _ reference file(s) [hash reference]
   #param4 _ remake reports? (1 - yes :: 0 - no)
   #param5 _ TARGET NAME (base filename)
   #param6 _ REF NAME (base filename)
   #param7 _ tools
   #param8 _ stemming? (0/1)
   #param9 _ verbosity (0/1)

   my $src = shift;
   my $out = shift;
   my $Href = shift;
   my $remakeREPORTS = shift;
   my $TGTname = shift;
   my $REFname = shift;
   my $tools = shift;
   my $stem = shift;
   my $verbose = shift;

   my $toolROUGE = "$tools/$ROUGE::TROUGE -e $tools/$ROUGE::TROUGEdata -z SPL -2 -1 -U -r 1000 -n 4 -w 1.2 -c 95 -d".($stem? " -m":"");

   my $reportROUGE = "./$COMMON::TMP/".rand($COMMON::NRAND).".$ROUGEXT.$COMMON::REPORTEXT";
   my $configROUGE = "./$COMMON::TMP/".rand($COMMON::NRAND).".$ROUGEXT.$ROUGE::CFGEXT";

   open(OUT, "< $out") or die "couldn't open file: $out\n";
   open(CFG, "> $configROUGE") or die "couldn't open file: $configROUGE\n";
   if ($verbose > 1) { print STDERR "generating files for ROUGE"; }
   my %hRAND;
   foreach my $r (keys %{$Href}) {
      my $ref = $Href->{$r};
      open(REF, "< $ref") or die "couldn't open file: $ref\n";
      my $j = 1;
      my $random = rand($COMMON::NRAND);
      $hRAND{$r} = $random;
      while (my $auxREF = <REF>) {
         chomp($auxREF);
         my $ref_j = "./$COMMON::TMP/$ROUGEXT.$random.$COMMON::REFEXT.$j";
         if (!(-e $ref_j) or $remakeREPORTS) {
            open(REFj, "> $ref_j") or die "couldn't open file: $ref_j\n";
            $auxREF =~ s/^ +//g; $auxREF =~ s/ +$//g;
            if (($auxREF eq "") or ($auxREF =~ /^[\?\!\.]$/)){ $auxREF = $COMMON::EMPTY_ITEM; }
            print REFj $auxREF, "\n";
            close(REFj);
         }
         $j++;
      }
      close(REF);
   }
   my $i = 1;
   my $random = rand($COMMON::NRAND);
   while (my $auxOUT = <OUT>) {
      chomp($auxOUT);
      my @cfgline;
      my $out_i = "./$COMMON::TMP/$ROUGEXT.$random.$COMMON::SYSEXT.$i";
      foreach my $r (keys %{$Href}) {
         my $ref_i = "./$COMMON::TMP/$ROUGEXT.".$hRAND{$r}.".$COMMON::REFEXT.$i";
         push(@cfgline, $ref_i);
      }
      if (!(-e $out_i) or $remakeREPORTS) {
         open(OUTi, "> $out_i") or die "couldn't open file: $out_i\n";
         $auxOUT =~ s/^ +//g; $auxOUT =~ s/ +$//g;
         if (($auxOUT eq "") or ($auxOUT =~ /^[\?\!\.]$/)) { $auxOUT = $COMMON::EMPTY_ITEM; }
         print OUTi $auxOUT, "\n";
         close(OUTi);
      }
      if ($verbose > 1) { 
         if ($i % 10 == 0) { print STDERR "."; }
         if ($i % 100 == 0) { print STDERR "$i"; }
      }
      $i++;
      print CFG "$out_i ", join(" ", @cfgline), "\n";
   }
   if ($verbose > 1) { print STDERR "..", $i-1, " segments [DONE]\n"; }
   close(CFG);
   close(OUT);

   if ($verbose > 1) { print STDERR "building $reportROUGE...\n"; }
   system("$toolROUGE $configROUGE > $reportROUGE");
   system("rm $configROUGE");
   my $j = 1;
   while ($j < $i) { if (-e "./$COMMON::TMP/$ROUGEXT.$random.$COMMON::SYSEXT.$j") { system("rm ./$COMMON::TMP/$ROUGEXT.$random.$COMMON::SYSEXT.$j"); } $j++; }
   foreach my $r (keys %{$Href}) {
      my $ref = $Href->{$r};
      $j = 1;
      while ($j < $i) { if (-e "./$COMMON::TMP/$ROUGEXT.".$hRAND{$r}.".$COMMON::REFEXT.$j") { system("rm ./$COMMON::TMP/$ROUGEXT.".$hRAND{$r}.".$COMMON::REFEXT.$j"); } $j++; }
   }
   my $SYS = read_rouge($reportROUGE);
   my $SEG = read_rouge_segments($reportROUGE);
   system("rm $reportROUGE");

   return ($SYS, $SEG);
}

sub doMultiROUGE {
   #description _ computes ROUGE scores -> n = 1..4, LCS, S*, SU*, W-1.2 (multiple references)
   #param1  _ source file
   #param2  _ candidate file
   #param3  _ reference file(s) [hash reference]
   #param4  _ remake reports? (1 - yes :: 0 - no)
   #param5  _ TARGET NAME (base filename)
   #param6  _ REF NAME (base filename)
   #param7  _ tools
   #param8  _ metrics to compute
   #param9  _ do stemming? (0/1)
   #param10 _ optional prefix
   #param11 _ verbosity (0/1)

   my $src = shift;
   my $out = shift;
   my $Href = shift;
   my $remakeREPORTS = shift;
   my $TGTname = shift;
   my $REFname = shift;
   my $tools = shift;
   my $M = shift;
   my $stemming = shift;
   my $prefix = shift;
   my $verbose = shift;

   my $GO = 0; my $i = 0;
   my @mROUGE = keys %{$ROUGE::rROUGE};
   while (($i < scalar(@mROUGE)) and (!$GO)) { if (exists($M->{$prefix.$mROUGE[$i]})) { $GO = 1; } $i++; }

   if ($GO) {
      if ($verbose == 1) { print STDERR "ROUGE.."; }
      my $reportROUGE1xml = "./$TGTname/$REFname/$prefix$ROUGEXT-1.$COMMON::XMLEXT";
      my $reportROUGE2xml = "./$TGTname/$REFname/$prefix$ROUGEXT-2.$COMMON::XMLEXT";
      my $reportROUGE3xml = "./$TGTname/$REFname/$prefix$ROUGEXT-3.$COMMON::XMLEXT";
      my $reportROUGE4xml = "./$TGTname/$REFname/$prefix$ROUGEXT-4.$COMMON::XMLEXT";
      my $reportROUGELxml = "./$TGTname/$REFname/$prefix$ROUGEXT-L.$COMMON::XMLEXT";
      my $reportROUGESxml = "./$TGTname/$REFname/$prefix$ROUGEXT-S*.$COMMON::XMLEXT";
      my $reportROUGESUxml = "./$TGTname/$REFname/$prefix$ROUGEXT-SU*.$COMMON::XMLEXT";
      my $reportROUGEWxml = "./$TGTname/$REFname/$prefix$ROUGEXT-W-1.2.$COMMON::XMLEXT";

      if ((!(-e $reportROUGE1xml) and !(-e $reportROUGE1xml.".$COMMON::GZEXT")) or (!(-e $reportROUGE2xml) and !(-e $reportROUGE2xml.".$COMMON::GZEXT")) or (!(-e $reportROUGE3xml) and !(-e $reportROUGE3xml.".$COMMON::GZEXT")) or (!(-e $reportROUGE4xml) and !(-e $reportROUGE4xml.".$COMMON::GZEXT")) or (!(-e $reportROUGELxml) and !(-e $reportROUGELxml.".$COMMON::GZEXT")) or (!(-e $reportROUGESxml) and !(-e $reportROUGESxml.".$COMMON::GZEXT")) or (!(-e $reportROUGESUxml) and !(-e $reportROUGESUxml.".$COMMON::GZEXT")) or (!(-e $reportROUGEWxml) and !(-e $reportROUGEWxml.".$COMMON::GZEXT")) or $remakeREPORTS) {
         my ($SYS, $SEG) = ROUGE::computeMultiROUGE($src, $out, $Href, $remakeREPORTS, $TGTname, $REFname, $tools, $stemming, $verbose);
         IQXML::write_report($TGTname, $REFname, "$prefix$ROUGEXT-1", $SYS->[0], $SEG->[0], $reportROUGE1xml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$prefix$ROUGEXT-2", $SYS->[1], $SEG->[1], $reportROUGE2xml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$prefix$ROUGEXT-3", $SYS->[2], $SEG->[2], $reportROUGE3xml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$prefix$ROUGEXT-4", $SYS->[3], $SEG->[3], $reportROUGE4xml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$prefix$ROUGEXT-L", $SYS->[4], $SEG->[4], $reportROUGELxml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$prefix$ROUGEXT-S*", $SYS->[5], $SEG->[5], $reportROUGESxml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$prefix$ROUGEXT-SU*", $SYS->[6], $SEG->[6], $reportROUGESUxml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$prefix$ROUGEXT-W-1.2", $SYS->[7], $SEG->[7], $reportROUGEWxml, $verbose);
      }
   }
}

sub computeROUGE {
   #description _ computes ROUGE scores -> n = 1..4, LCS, S*, SU*, W-1.2 (single reference)
   #param1 _ source file
   #param2 _ candidate file
   #param3 _ reference file 
   #param4 _ remake reports? (1 - yes :: 0 - no)
   #param5 _ target NAME (base filename)
   #param6 _ reference NAME (base filename)
   #param7 _ tools
   #param8 _ stemming? (0/1)
   #param9 _ verbosity (0/1)

   my $src = shift;
   my $out = shift;
   my $ref = shift;
   my $remakeREPORTS = shift;
   my $TGTname = shift;
   my $REFname = shift;
   my $tools = shift;
   my $stem = shift;
   my $verbose = shift;

   my $toolROUGE = "$tools/$ROUGE::TROUGE -e $tools/$ROUGE::TROUGEdata -z SPL -2 -1 -U -r 1000 -n 4 -w 1.2 -c 95 -d".($stem? " -m":"");

   my $configROUGE = "./$COMMON::TMP/".rand($COMMON::NRAND).".$ROUGEXT.$ROUGE::CFGEXT";
   my $reportROUGE = "./$COMMON::TMP/".rand($COMMON::NRAND).".$ROUGEXT.$COMMON::REPORTEXT";

   open(OUT, "< $out") or die "couldn't open file: $out\n";
   open(REF, "< $ref") or die "couldn't open file: $ref\n";
   open(CFG, "> $configROUGE") or die "couldn't open file: $configROUGE\n";
   my $i = 1;
   my $random = rand($COMMON::NRAND);
   while (my $auxREF = <REF>) {
      my $auxOUT = <OUT>;
      chomp($auxREF);
      chomp($auxOUT);
      my $out_i = "./$COMMON::TMP/$ROUGEXT.$random.$COMMON::SYSEXT.$i";
      my $ref_i = "./$COMMON::TMP/$ROUGEXT.$random.$COMMON::REFEXT.$i";
      if (!(-e $ref_i) or $remakeREPORTS) {
         open(REFi, "> $ref_i") or die "couldn't open file: $ref_i\n";
         $auxREF =~ s/^ +//g; $auxREF =~ s/ +$//g;
         if (($auxREF eq "") or ($auxREF =~ /^[\?\!\.]$/)){ $auxREF = $COMMON::EMPTY_ITEM; }
         print REFi $auxREF, "\n";
         close(REFi);
      }
      if (!(-e $out_i) or $remakeREPORTS) {
         open(OUTi, "> $out_i") or die "couldn't open file: $out_i\n";
         $auxOUT =~ s/^ +//g; $auxOUT =~ s/ +$//g;
         if (($auxOUT eq "") or ($auxOUT =~ /^[\?\!\.]$/)) { $auxOUT = $COMMON::EMPTY_ITEM; }
         print OUTi $auxOUT, "\n";
         close(OUTi);
      }
      print CFG "$out_i $ref_i\n";
      $i++;
   }
   close(CFG);
   close(REF);
   close(OUT);
   if ($verbose > 1) { print STDERR "building $reportROUGE...\n"; }
   system("$toolROUGE $configROUGE > $reportROUGE");
   system("rm $configROUGE");
   my $j = 1;
   while ($j < $i) { if ((-e "./$COMMON::TMP/$ROUGEXT.$random.$COMMON::REFEXT.$j")) { system("rm ./$COMMON::TMP/$ROUGEXT.$random.$COMMON::REFEXT.$j"); } $j++; }
   $j = 1;
   while ($j < $i) { if ((-e "./$COMMON::TMP/$ROUGEXT.$random.$COMMON::SYSEXT.$j")) { system("rm ./$COMMON::TMP/$ROUGEXT.$random.$COMMON::SYSEXT.$j"); } $j++; }
   my $SYS = read_rouge($reportROUGE);
   my $SEG = read_rouge_segments($reportROUGE);
   system("rm $reportROUGE");

   return ($SYS, $SEG);
}

sub doROUGE {
   #description _ computes ROUGE scores -> n = 1..4, LCS, S*, SU*, W-1.2 (single reference)
   #param1  _ source file
   #param2  _ candidate file
   #param3  _ reference file(s) [hash reference]
   #param4  _ remake reports? (1 - yes :: 0 - no)
   #param5  _ TARGET NAME (base filename)
   #param6  _ REF NAME (base filename)
   #param7  _ tools
   #param8  _ metrics to compute
   #param9  _ do stemming? (0/1)
   #param10 _ optional prefix
   #param11 _ verbosity (0/1)

   my $src = shift;
   my $out = shift;
   my $ref = shift;
   my $remakeREPORTS = shift;
   my $TGTname = shift;
   my $REFname = shift;
   my $tools = shift;
   my $M = shift;
   my $stemming = shift;
   my $prefix = shift;
   my $verbose = shift;

   my $GO = 0; my $i = 0;
   my @mROUGE = keys %{$ROUGE::rROUGE};
   while (($i < scalar(@mROUGE)) and (!$GO)) { if (exists($M->{$prefix.$mROUGE[$i]})) { $GO = 1; } $i++; }

   if ($GO) {
      if ($verbose == 1) { print STDERR "ROUGE.."; }
      my $reportROUGE1xml = "./$TGTname/$REFname/$prefix$ROUGEXT-1.$COMMON::XMLEXT";
      my $reportROUGE2xml = "./$TGTname/$REFname/$prefix$ROUGEXT-2.$COMMON::XMLEXT";
      my $reportROUGE3xml = "./$TGTname/$REFname/$prefix$ROUGEXT-3.$COMMON::XMLEXT";
      my $reportROUGE4xml = "./$TGTname/$REFname/$prefix$ROUGEXT-4.$COMMON::XMLEXT";
      my $reportROUGELxml = "./$TGTname/$REFname/$prefix$ROUGEXT-L.$COMMON::XMLEXT";
      my $reportROUGESxml = "./$TGTname/$REFname/$prefix$ROUGEXT-S*.$COMMON::XMLEXT";
      my $reportROUGESUxml = "./$TGTname/$REFname/$prefix$ROUGEXT-SU*.$COMMON::XMLEXT";
      my $reportROUGEWxml = "./$TGTname/$REFname/$prefix$ROUGEXT-W-1.2.$COMMON::XMLEXT";

      if ((!(-e $reportROUGE1xml) and !(-e $reportROUGE1xml.".$COMMON::GZEXT")) or (!(-e $reportROUGE2xml) and !(-e $reportROUGE2xml.".$COMMON::GZEXT")) or (!(-e $reportROUGE3xml) and !(-e $reportROUGE3xml.".$COMMON::GZEXT")) or (!(-e $reportROUGE4xml) and !(-e $reportROUGE4xml.".$COMMON::GZEXT")) or (!(-e $reportROUGELxml) and !(-e $reportROUGELxml.".$COMMON::GZEXT")) or (!(-e $reportROUGESxml) and !(-e $reportROUGESxml.".$COMMON::GZEXT")) or (!(-e $reportROUGESUxml) and !(-e $reportROUGESUxml.".$COMMON::GZEXT")) or (!(-e $reportROUGEWxml) and !(-e $reportROUGEWxml.".$COMMON::GZEXT")) or $remakeREPORTS) {
         my ($SYS, $SEG) = ROUGE::computeROUGE($src, $out, $ref, $remakeREPORTS, $TGTname, $REFname, $tools, $stemming, $verbose);
         IQXML::write_report($TGTname, $REFname, "$prefix$ROUGEXT-1", $SYS->[0], $SEG->[0], $reportROUGE1xml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$prefix$ROUGEXT-2", $SYS->[1], $SEG->[1], $reportROUGE2xml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$prefix$ROUGEXT-3", $SYS->[2], $SEG->[2], $reportROUGE3xml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$prefix$ROUGEXT-4", $SYS->[3], $SEG->[3], $reportROUGE4xml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$prefix$ROUGEXT-L", $SYS->[4], $SEG->[4], $reportROUGELxml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$prefix$ROUGEXT-S*", $SYS->[5], $SEG->[5], $reportROUGESxml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$prefix$ROUGEXT-SU*", $SYS->[6], $SEG->[6], $reportROUGESUxml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$prefix$ROUGEXT-W-1.2", $SYS->[7], $SEG->[7], $reportROUGEWxml, $verbose);
      }
   }
}

1;
